
#include <proto/exec.h>
#include <proto/dos.h>
#include <dos/dosextens.h>
#include <string.h>


struct pathnode {
	BPTR next;
	BPTR path;
	};


BPTR LockFile (char *name) /* Lock() only files */

{
BPTR lock;
struct FileInfoBlock *fib;

lock = Lock (name,SHARED_LOCK);

if (lock)
	if (fib = AllocVec (sizeof(struct FileInfoBlock),MEMF_CLEAR))
		{
		if (Examine(lock,fib))
			if (fib->fib_DirEntryType > 0)
				{
				UnLock (lock);
				lock = (BPTR)0;  /* don't allow directories */
				}

		FreeVec (fib);
		}

return (lock);
}


BPTR check_dir (BPTR dir,char *file) /* check if file is in directory */

{
BPTR oldcd;
BPTR lock;

oldcd = CurrentDir (dir);
lock = LockFile (file);
CurrentDir (oldcd);

return (lock);
}


BPTR which (UBYTE *command) /* find command in DOS path */

{
BPTR cmdlock;

cmdlock = LockFile (command); /* look in current directory */

if (!cmdlock && command == FilePart(command)) /* check DOS path only if no path is given in command name */
	{
	struct CommandLineInterface *cli;
	struct pathnode *n;

	if (cli = Cli())
		for (n = BADDR(cli->cli_CommandDir); !cmdlock && n; n = BADDR(n->next))
			cmdlock = check_dir (n->path,command);

	if (!cmdlock) /* C: is not contained in the path list */
		{
		struct DosList *dl;
		struct AssignList *al;

		dl = LockDosList (LDF_ASSIGNS|LDF_READ);
		if (dl = FindDosEntry (dl,"C",LDF_ASSIGNS)) /* because C: may be a multi-assign we cannot just Lock("C:") */
			{
			cmdlock = check_dir (dl->dol_Lock,command);
			for (al = dl->dol_misc.dol_assign.dol_List; !cmdlock && al; al = al->al_Next)
				cmdlock = check_dir (al->al_Lock,command);
			}
		UnLockDosList (LDF_ASSIGNS|LDF_READ);
		}
	}

return (cmdlock); /* whatever we found or NULL */
}


int main (void)

{
struct RDArgs *rdargs;
struct {
	char *file;
	} args = {NULL};
BPTR lock;
char buffer[256];

if (rdargs = ReadArgs ("FILE/A",(APTR)&args,NULL))
	{

	if (lock = which (args.file))
		{
		NameFromLock (lock,buffer,256);
		Printf ("%s\n",buffer);
		UnLock (lock);
		}

	FreeArgs (rdargs);
	}

PrintFault (IoErr(),NULL);

return (0);
}
